-- FuBar_ChimeClockFu 1.5
-- Author: steste
-- Purpose: A simple chime clock for FuBar

ChimeClockFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

local FMT_MIL = 0 -- 24 hour format
local FMT_NORM = 1 -- 12 hour format
local DISPLAY_LOCAL = 0 -- display local time
local DISPLAY_SERVER = 1 -- display server time
local DISPLAY_BOTH = 2 -- display both times

ChimeClockFu.hour = 0
ChimeClockFu.minute = 0
ChimeClockFu.lastalarmmin = 0

-- Register the saved variable DB
ChimeClockFu:RegisterDB("ChimeClockFuDB")

-- Realm defaults for saved variables
ChimeClockFu:RegisterDefaults('server', {
	offset = 0, -- the offset from server to local time
	hourfmt = FMT_NORM,
	alarmpm = false,
	alarmpm2 = false,
	chimeenable = true,
	alarmenable = false,
	repeat_alarm = true,
	alarmenable_2 = false,
	repeat_alarm2 = true,
	alarmmin = 0,
	alarmmin_2 = 0,
	alarmhr = 0,
	alarmhr_2 = 0,
	alarmhrpre = 0,
	alarmhrpre_2 = 0,
	alarmmsg = "Alarm Activated!",
	alarmmsg_2 = "Alarm 2 Activated!",
	display = DISPLAY_LOCAL
})

-- Menu Request
ChimeClockFu.OnMenuRequest = {
	type = 'group',
	args = {
		chimeenable = { order = 30,
		    type = 'toggle',
			name = "Hourly Chime",
			desc = "Switches hourly chime off and on",
			set = "SetChime",
			get = "GetChime"
		},	
		offset = { order = 60,
			type = 'range',
			name = "Offset",
			desc = "Time offset from server time to local time",
			get = "GetOffset",
			set = "SetOffset",
			min = -12,
			max = 12,
			step = 1
		},
		hourfmt = { order = 50,
			type = 'group',
			desc = "Select a time format",
			name = "Hour Format",
			args = {
				normalfmt = {
					type = 'toggle',
					name = "12 Hour Format",
					desc = "Select a 12 hour time format",
					set = "SetNormalFormat",
					get = "IsNormalFormat"
				},
				militaryfmt = {
					type = 'toggle',
					name = "24 Hour Format",
					desc = "Select a 24 hour time format",
					set = "SetMilitaryFormat",
					get = "IsMilitaryFormat"
				}
			}
		},
		display = { order = 40,
			type = 'group',
			name = "Display Settings",
			desc = "Choose the time that will be displayed",
			args = {
				disp_local = {
					type = 'toggle',
					name = "Local Time",
					desc = "Display only local time",
					get = "IsLocalFormat",
					set = "SetLocalFormat"
				},
				disp_server = {
					type = 'toggle',
					name = "Server Time",
					desc = "Display only server time",
					get = "IsServerFormat",
					set = "SetServerFormat"
				},
				disp_both = {
					type = 'toggle',
					name = "Show Both",
					desc = "Display both times",
					get = "IsBothFormat",
					set = "SetBothFormat"
				}
			}
		},
		alarm = { order = 10,
			type = 'group',
			name = "Alarm Settings",
			desc = "Change alarm settings",
			args = {
				alarmenable = {
					type = 'toggle',
					name = "Enable Alarm",
					desc = "Switch the alarm on or off",
					get = "IsAlarmEnabled",
					set = "SetAlarmEnabled"
				},
				repeat_alarm = {
					type = 'toggle',
					name = "Recurring Alarm",
					desc = "Toggle repeating this alarm",
					get = "IsAlarmRepeating",
					set = "SetAlarmRepeating"
				},
				alarmtime = {
					type = 'group',
					name = "Alarm Time",
					desc = "Set the alarm time",
					args = {
						alarmhr = {
							type = 'text',
							name = "Alarm Hour",
							desc = "Set the alarm hour",
							usage = "<hour>",
							get = "GetAlarmHour",
							set = "SetAlarmHour"
						},
						alarmmin = {
							type = 'text',
							name = "Alarm Minute",
							desc = "Set the alarm minute",
							usage = "<minute>",
							get = "GetAlarmMinute",
							set = "SetAlarmMinute"
						},
						alarmampm = {
							type = 'toggle',
							name = "Pm",
							hidden = "IsMilitaryFormat",
							desc = "Sets Pm for alarm",
							get = "IsAlarmPm",
							set = "SetAlarmPm",
						}
					}
				},
				alarmmsg = {
					type = 'text',
					name = "Alarm Message",
					desc = "Set the message displayed when the alarm goes off",
					usage = "<alarm message>",
					get = "GetAlarmMessage",
					set = "SetAlarmMessage"
				}
			}
		},
		alarm_2 = { order = 20,
			type = 'group',
			name = "Alarm 2 Settings",
			desc = "Change alarm settings",
			args = {
				alarmenable = {
					type = 'toggle',
					name = "Enable Alarm",
					desc = "Switch the alarm on or off",
					get = "IsAlarm2Enabled",
					set = "SetAlarm2Enabled"
				},
				repeat_alarm = {
					type = 'toggle',
					name = "Recurring Alarm",
					desc = "Toggle repeating this alarm",
					get = "IsAlarm2Repeating",
					set = "SetAlarm2Repeating"
				},
				alarmtime = {
					type = 'group',
					name = "Alarm Time",
					desc = "Set the alarm time",
					args = {
						alarmhr = {
							type = 'text',
							name = "Alarm Hour",
							desc = "Set the alarm hour",
							usage = "<hour>",
							get = "GetAlarm2Hour",
							set = "SetAlarm2Hour"
						},
						alarmmin = {
							type = 'text',
							name = "Alarm Minute",
							desc = "Set the alarm minute",
							usage = "<minute>",
							get = "GetAlarm2Minute",
							set = "SetAlarm2Minute"
						},
						alarmampm = {
							type = 'toggle',
							name = "Pm",
							hidden = "IsMilitaryFormat",
							desc = "Sets Pm for alarm",
							get = "IsAlarmPm2",
							set = "SetAlarmPm2",
						}
					}
				},
				alarmmsg = {
					type = 'text',
					name = "Alarm Message",
					desc = "Set the message displayed when the alarm goes off",
					usage = "<alarm message>",
					get = "GetAlarm2Message",
					set = "SetAlarm2Message"
				}
			}
		}
	}
}

-- Get / Set Methods
function ChimeClockFu:IsLocalFormat()
	if (self.db.server.display == DISPLAY_LOCAL) then
		return true
	end
	
	return false
end

function ChimeClockFu:IsServerFormat()
	if (self.db.server.display == DISPLAY_SERVER) then
		return true
	end
	
	return false
end

function ChimeClockFu:IsBothFormat()
	if (self.db.server.display == DISPLAY_BOTH) then
		return true
	end
	
	return false
end

function ChimeClockFu:SetLocalFormat()
	self.db.server.display = DISPLAY_LOCAL
end

function ChimeClockFu:SetServerFormat()
	self.db.server.display = DISPLAY_SERVER
end

function ChimeClockFu:SetBothFormat()
	self.db.server.display = DISPLAY_BOTH
end
	
function ChimeClockFu:GetOffset()
	return self.db.server.offset
end

function ChimeClockFu:SetOffset(v)
	self.db.server.offset = v
end

function ChimeClockFu:IsMilitaryFormat()
	if self.db.server.hourfmt == FMT_MIL then
		return true
	end
	
	return false
end

function ChimeClockFu:IsNormalFormat()
	if self.db.server.hourfmt == FMT_NORM then
		return true
	end
	
	return false
end

function ChimeClockFu:SetMilitaryFormat()
	self.db.server.hourfmt = FMT_MIL
end

function ChimeClockFu:SetNormalFormat()
	self.db.server.hourfmt = FMT_NORM
end

function ChimeClockFu:GetAlarmHour()
	return self.db.server.alarmhrpre
end

function ChimeClockFu:SetAlarmHour(hour)
	self.db.server.alarmhrpre = tonumber(hour)
end

function ChimeClockFu:GetAlarm2Hour()
	return self.db.server.alarmhrpre_2
end

function ChimeClockFu:SetAlarm2Hour(hour)
	self.db.server.alarmhrpre_2 = tonumber(hour)
end

function ChimeClockFu:GetAlarmMinute()
	return self.db.server.alarmmin
end

function ChimeClockFu:SetAlarmMinute(min)
	self.db.server.alarmmin = tonumber(min)
end

function ChimeClockFu:GetAlarm2Minute()
	return self.db.server.alarmmin_2
end

function ChimeClockFu:SetAlarm2Minute(min)
	self.db.server.alarmmin_2 = tonumber(min)
end

function ChimeClockFu:GetChime()
	return self.db.server.chimeenable
end

function ChimeClockFu:SetChime()
	self.db.server.chimeenable = not self.db.server.chimeenable
end

function ChimeClockFu:IsAlarmEnabled()
	return self.db.server.alarmenable
end

function ChimeClockFu:SetAlarmEnabled()
	self.db.server.alarmenable = not self.db.server.alarmenable
end

function ChimeClockFu:IsAlarmPm2()
	return self.db.server.alarmpm2
end

function ChimeClockFu:SetAlarmPm2()
	self.db.server.alarmpm2 = not self.db.server.alarmpm2
end

function ChimeClockFu:IsAlarmPm()
	return self.db.server.alarmpm
end

function ChimeClockFu:SetAlarmPm()
	self.db.server.alarmpm = not self.db.server.alarmpm
end

function ChimeClockFu:IsAlarmRepeating()
	return self.db.server.repeat_alarm
end

function ChimeClockFu:SetAlarmRepeating()
	self.db.server.repeat_alarm = not self.db.server.repeat_alarm
end

function ChimeClockFu:IsAlarm2Enabled()
	return self.db.server.alarmenable_2
end

function ChimeClockFu:SetAlarm2Enabled()
	self.db.server.alarmenable_2 = not self.db.server.alarmenable_2
end

function ChimeClockFu:IsAlarm2Repeating()
	return self.db.server.repeat_alarm2
end

function ChimeClockFu:SetAlarm2Repeating()
	self.db.server.repeat_alarm2 = not self.db.server.repeat_alarm2
end

function ChimeClockFu:GetAlarmMessage()
	return self.db.server.alarmmsg
end

function ChimeClockFu:SetAlarmMessage(msg)
	self.db.server.alarmmsg = msg
end

function ChimeClockFu:GetAlarm2Message()
	return self.db.server.alarmmsg_2
end

function ChimeClockFu:SetAlarm2Message(msg)
	self.db.server.alarmmsg_2 = msg
end

-- Formats the text displayed for the addon
function ChimeClockFu:FormatClockString()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour

	hour = hour + self.db.server.offset
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.server.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

-- used to get the server time
function ChimeClockFu:FormatClockNoOffset()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.server.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

-- formats the alarm time
function ChimeClockFu:FormatAlarmString()
	local hour = self.db.server.alarmhr
	local minute = self.db.server.alarmmin
	local hrpre = self.db.server.alarmhrpre

	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.server.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

function ChimeClockFu:FormatAlarm2String()
	local hour = self.db.server.alarmhr_2
	local minute = self.db.server.alarmmin_2

	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.server.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

function ChimeClockFu:GetNextMinute(curmin)
	if (curmin == 60) then
		return 0
	else
		return curmin + 1
	end
end

-- Called when the addon is initialized
function ChimeClockFu:OnInitialize()
	self:ScheduleRepeatingEvent(self.OnUpdate, 1, self)
	self:RegisterEvent("ChimeClockFu_ChimeTrigger", self.ChimeChimeClockTrigger, true)
	self:RegisterEvent("ChimeClockFu_AlarmTrigger", self.ChimeClockTrigger, true)
	self:RegisterEvent("ChimeClockFu_AlarmTrigger2", self.ChimeClockTrigger2, true)
end

-- Sets the displayed text
function ChimeClockFu:OnTextUpdate()
	self.hour, self.minute = GetGameTime()
	
	local timestr = ""
	if (self.db.server.display == DISPLAY_LOCAL) then
		timestr, _, _ = self:FormatClockString()
	elseif (self.db.server.display == DISPLAY_SERVER) then
		timestr, _, _ = self:FormatClockNoOffset()
	else
		local tempstr = ""
		tempstr, _, _ = self:FormatClockString()
		timestr, _, _ = self:FormatClockNoOffset()
		timestr = tempstr .. " | " .. timestr
	end
	
	self:SetText(timestr)
end

-- Make sure we update the text
-- Also, trigger alarms if need bew
function ChimeClockFu:OnUpdate()
	self:UpdateText()
	
	local oldformat = self.db.server.hourfmt

	if (oldformat == FMT_NORM) then
		self.db.server.hourfmt = FMT_MIL
	end
	
	local _, hour, min = self:FormatClockString()
	
	self.db.server.hourfmt = oldformat
	
	if (self:GetNextMinute(self.lastalarmmin) == self.minute) then
		self:RegisterEvent("ChimeClockFu_AlarmTrigger", self.ChimeClockTrigger)
	end
	
	if (self:GetNextMinute(self.lastalarmmin) == self.minute) then
		self:RegisterEvent("ChimeClockFu_AlarmTrigger2", self.ChimeClockTrigger2)
	end
	
	-- is chime ready?
	if (self.db.server.chimeenable == true) then
		if  (min == 00) then
			self:TriggerEvent("ChimeClockFu_ChimeTrigger")
		end
    end
	
	-- is alarm 1 ready?
	if (self.db.server.alarmenable == true) then
		if (hour == self.db.server.alarmhr and min == self.db.server.alarmmin) then
			self:TriggerEvent("ChimeClockFu_AlarmTrigger")
			
			-- check if the alarm is not recurring
			if (self.db.server.repeat_alarm == false) then
				self.db.server.alarmenable = false
			end
			
			self.lastalarmmin = self.minute
			if (self:IsEventRegistered("ChimeClockFu_AlarmTrigger")) then
				self:UnregisterEvent("ChimeClockFu_AlarmTrigger")
			end
		end
	end
	
	-- is alarm 2 ready?
	if (self.db.server.alarmenable_2 == true) then
		if (hour == self.db.server.alarmhr_2 and min == self.db.server.alarmmin_2) then
			self:TriggerEvent("ChimeClockFu_AlarmTrigger2")
			self.lastalarmmin = self.minute
			
			-- check if the alarm is not recurring
			if (self.db.server.repeat_alarm2 == false) then
				self.db.server.alarmenable_2 = false
			end
			
			if (self:IsEventRegistered("ChimeClockFu_AlarmTrigger2")) then
				self:UnregisterEvent("ChimeClockFu_AlarmTrigger2")
			end
		end
	end
	
	if (self.db.server.hourfmt == FMT_MIL) then
		self.db.server.alarmpm2 = false
	end
	
	if (self.db.server.hourfmt == FMT_MIL) then
		self.db.server.alarmpm = false
	end
	
	if (self.db.server.alarmpm == false) then
		self.db.server.alarmhr = self.db.server.alarmhrpre + 0
	end
	
	if (self.db.server.alarmpm == true) then
		self.db.server.alarmhr = self.db.server.alarmhrpre + 12
	end
	
	if (self.db.server.alarmpm2 == false) then
		self.db.server.alarmhr_2 = self.db.server.alarmhrpre_2 + 0
	end
	
	if (self.db.server.alarmpm2 == true) then
		self.db.server.alarmhr_2 = self.db.server.alarmhrpre_2 + 12
	end
end

-- Event handler for ChimeClockFu_AlarmTrigger event
function ChimeClockFu:ChimeChimeClockTrigger()
	PlaySoundFile("Interface\\AddOns\\FuBar_ChimeClockFu\\Chime.wav")
end

function ChimeClockFu:ChimeClockTrigger()
	_ERRORMESSAGE(ChimeClockFu.db.server.alarmmsg)
end

function ChimeClockFu:ChimeClockTrigger2()
	_ERRORMESSAGE(ChimeClockFu.db.server.alarmmsg_2)
end

local tablet = AceLibrary("Tablet-2.0")
function ChimeClockFu:OnTooltipUpdate()
	local timestr, _, _ = self:FormatClockString()
	local timestr_no_offset, _, _ = self:FormatClockNoOffset()
	local alarmtime, _, _ = self:FormatAlarmString()
	local alarm2time, _, _ = self:FormatAlarm2String()

	-- Add a date line to the tooltip
	tablet:AddCategory():AddLine(
		'text', date("%A, %B %d, %Y"),
		'textR', 1,
		'textG', 1,
		'textB', 1,
		'justify', "CENTER"
	)
	
	cat = tablet:AddCategory(
		'text', "Time",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', 1,
		'child_textG2', 1,
		'child_textB2', 1
	)
	
	cat:AddLine(
		'text', "Local Time",
		'text2', timestr
	)
	
	cat:AddLine(
		'text', "Server Time",
		'text2', timestr_no_offset
	)
	
	local offsetstr = ""
	
	if (self.db.server.offset > 0) then
		offsetstr = "+" .. self.db.server.offset
	else
		offsetstr = tostring(self.db.server.offset)
	end
	
	cat:AddLine(
		'text', "Offset",
		'text2', offsetstr
	)
	
	local r, g, b
	
	if (self.db.server.alarmenable == false) then
		r = 1
		g = 0
		b = 0
	else
		r = 0
		g = 1
		b = 0
	end
	
	cat = tablet:AddCategory(
		'text', "Chime",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', r,
		'child_textG2', g,
		'child_textB2', b
	)
	
	local enabled
	
	if self.db.server.chimeenable == false then
		enabled = "No"
	else
		enabled = "Yes"
	end
	
	cat:AddLine(
		'text', "Enabled",
		'text2', enabled
	)
	
	cat = tablet:AddCategory(
		'text', "Alarm 1",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', r,
		'child_textG2', g,
		'child_textB2', b
	)
	
	local enabled_1
	
	if self.db.server.alarmenable == false then
		enabled_1 = "No"
	else
		enabled_1 = "Yes"
	end
	
	cat:AddLine(
		'text', "Enabled",
		'text2', enabled_1
	)
	
	if self.db.server.alarmenable == true then
		cat:AddLine(
			'text', "Alarm 1 Time",
			'text2', alarmtime
		)
	end
	
	cat = tablet:AddCategory(
		'text', "Alarm 2",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', r,
		'child_textG2', g,
		'child_textB2', b
	)
	
	local enabled_2
	
	if (self.db.server.alarmenable_2 == false) then
		enabled_2 = "No"
	else
		enabled_2 = "Yes"
	end
	
	cat:AddLine(
		'text', "Enabled",
		'text2', enabled_2
	)
	
	if (self.db.server.alarmenable_2 == true) then
		cat:AddLine(
			'text', "Alarm 2 Time",
			'text2', alarm2time
		)
	end
end